import { jsx, jsxs } from 'react/jsx-runtime';
import { useState, useSyncExternalStore } from 'react';
import PropTypes from 'prop-types';

const stringOrElement = PropTypes.oneOfType([PropTypes.string, PropTypes.element]);
const objectOfStringOrElement = PropTypes.objectOf(stringOrElement);
const getClassNameOrStyleProps = (classNameOrStyle, narrow) => {
    if (!classNameOrStyle) {
        return {};
    }
    if (typeof classNameOrStyle === 'function') {
        classNameOrStyle = classNameOrStyle({ narrow });
    }
    if (typeof classNameOrStyle === 'string') {
        return { className: classNameOrStyle };
    }
    if (typeof classNameOrStyle === 'object') {
        return { style: classNameOrStyle };
    }
    return {};
};
function headerClass(withClasses, key) {
    return withClasses ? { className: `header-${key}` } : {};
}
function rowClass(withClasses, key) {
    return withClasses ? { className: `row-${key}` } : {};
}
const inBrowser = typeof window !== 'undefined';
const matchMedia = inBrowser && window.matchMedia !== null;
const initialState = { subscriber: () => () => ({}) };
const HyperResponsiveTable = ({ headers, rows, breakpoint, keyGetter, tableStyling, initialNarrow, withClasses, }) => {
    const [state, setState] = useState(initialState);
    const [oldBreakpoint, setOldBreakpoint] = useState();
    if (oldBreakpoint !== breakpoint && matchMedia) {
        const mql = window.matchMedia(typeof breakpoint === 'string' ? breakpoint : `(min-width: ${breakpoint}px)`);
        setState({
            mql,
            subscriber: onStoreChange => {
                if (matchMedia) {
                    mql.addListener(onStoreChange);
                    return () => mql.removeListener(onStoreChange);
                }
                return () => ({});
            },
        });
        setOldBreakpoint(breakpoint);
    }
    const narrow = useSyncExternalStore(state.subscriber, () => (state.mql ? !state.mql.matches : false), // on client
    () => initialNarrow // on server
    );
    const dataKeys = Object.keys(headers);
    if (narrow) {
        return (jsx("table", Object.assign({}, getClassNameOrStyleProps(tableStyling, narrow), { children: rows.map(row => (jsx("tbody", { children: dataKeys.map(key => (jsxs("tr", Object.assign({}, rowClass(withClasses, keyGetter(row)), { children: [jsx("th", Object.assign({}, headerClass(withClasses, key), { scope: "row" }, { children: headers[key] })), jsx("td", { children: row[key] })] }), key))) }, keyGetter(row)))) })));
    }
    return (jsxs("table", Object.assign({}, getClassNameOrStyleProps(tableStyling, narrow), { children: [jsx("thead", { children: jsx("tr", { children: dataKeys.map(key => (jsx("th", Object.assign({}, headerClass(withClasses, key), { scope: "col" }, { children: headers[key] }), key))) }) }), jsx("tbody", { children: rows.map(row => (jsx("tr", Object.assign({}, rowClass(withClasses, keyGetter(row)), { children: dataKeys.map(key => (jsx("td", { children: row[key] }, key))) }), keyGetter(row)))) })] })));
};
HyperResponsiveTable.propTypes = {
    headers: objectOfStringOrElement.isRequired,
    rows: PropTypes.arrayOf(objectOfStringOrElement).isRequired,
    breakpoint: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
    keyGetter: PropTypes.func.isRequired,
    tableStyling: PropTypes.oneOfType([PropTypes.string, PropTypes.object, PropTypes.func]),
    initialNarrow: PropTypes.bool,
    withClasses: PropTypes.bool,
};
HyperResponsiveTable.defaultProps = {
    initialNarrow: false,
    withClasses: false,
    tableStyling: null,
};

export { HyperResponsiveTable as default };
